@@ -6,6 +6,7 @@ from .llm_agent import LLMAgent
 from .llm_agent_memory import LLMAgentMemory
 from .memory import Memory
 from .packet import Packet
+from .printer import Printer, print_event_stream
 from .processors.base_processor import BaseProcessor
 from .processors.parallel_processor import ParallelProcessor
 from .processors.processor import Processor
@@ -33,9 +34,11 @@ __all__ = [
     "Packet",
     "Packet",
     "ParallelProcessor",
+    "Printer",
     "ProcName",
     "Processor",
     "RunContext",
     "SystemMessage",
     "UserMessage",
+    "print_event_stream",
 ]
