@@ -1,5 +1,3 @@
-from typing import Any
-
 import httpx
 from pydantic import BaseModel, NonNegativeFloat, PositiveInt
 
@@ -9,13 +7,26 @@ class AsyncHTTPClientParams(BaseModel):
     max_connections: PositiveInt = 2000
     max_keepalive_connections: PositiveInt = 500
     keepalive_expiry: float | None = 5
+    # proxy: str | None = None
+    # follow_redirects: bool = False
+    # trust_env: bool = True
+    # auth: AuthTypes | None = (None,)
+    # params: QueryParamTypes | None = (None,)
+    # headers: HeaderTypes | None = (None,)
+    # cookies: CookieTypes | None = (None,)
 
 
 def create_simple_async_httpx_client(
-    client_params: AsyncHTTPClientParams | dict[str, Any],
+    client_params: AsyncHTTPClientParams,
 ) -> httpx.AsyncClient:
-    if isinstance(client_params, dict):
-        client_params = AsyncHTTPClientParams(**client_params)
+    extra_params = client_params.model_dump(
+        exclude={
+            "timeout",
+            "max_connections",
+            "max_keepalive_connections",
+            "keepalive_expiry",
+        }
+    )
     return httpx.AsyncClient(
         timeout=httpx.Timeout(client_params.timeout),
         limits=httpx.Limits(
@@ -23,4 +34,5 @@ def create_simple_async_httpx_client(
             max_keepalive_connections=client_params.max_keepalive_connections,
             keepalive_expiry=client_params.keepalive_expiry,
         ),
+        **extra_params,
     )
