@@ -7,14 +7,14 @@ from typing import Any, Generic, Protocol, final
 
 from pydantic import BaseModel
 
+from grasp_agents.tracing_decorators import task
 from grasp_agents.typing.completion_chunk import CompletionChunk
 
 from .errors import AgentFinalAnswerError
-from .llm import LLM, LLMSettings
+from .llm import LLM
 from .llm_agent_memory import LLMAgentMemory
 from .run_context import CtxT, RunContext
 from .typing.completion import Completion
-from .typing.converters import Converters
 from .typing.events import (
     CompletionChunkEvent,
     CompletionEvent,
@@ -58,7 +58,7 @@ class LLMPolicyExecutor(Generic[CtxT]):
         self,
         *,
         agent_name: str,
-        llm: LLM[LLMSettings, Converters],
+        llm: LLM,
         tools: list[BaseTool[BaseModel, Any, CtxT]] | None,
         response_schema: Any | None = None,
         response_schema_by_xml_tag: Mapping[str, Any] | None = None,
@@ -96,7 +96,7 @@ class LLMPolicyExecutor(Generic[CtxT]):
         return self._agent_name
 
     @property
-    def llm(self) -> LLM[LLMSettings, Converters]:
+    def llm(self) -> LLM:
         return self._llm
 
     @property
@@ -147,6 +147,7 @@ class LLMPolicyExecutor(Generic[CtxT]):
         if self.memory_manager:
             self.memory_manager(memory=memory, ctx=ctx, call_id=call_id, **kwargs)
 
+    @task(name="generate_message")  # type: ignore
     async def generate_message(
         self,
         memory: LLMAgentMemory,
@@ -161,17 +162,15 @@ class LLMPolicyExecutor(Generic[CtxT]):
             response_schema_by_xml_tag=self.response_schema_by_xml_tag,
             tools=self.tools,
             tool_choice=tool_choice,
-            n_choices=1,
             proc_name=self.agent_name,
             call_id=call_id,
         )
         memory.update(completion.messages)
-        self._process_completion(
-            completion, ctx=ctx, call_id=call_id, print_messages=True
-        )
+        self._process_completion(completion, ctx=ctx, call_id=call_id)
 
         return completion.messages[0]
 
+    @task(name="generate_message")  # type: ignore
     async def generate_message_stream(
         self,
         memory: LLMAgentMemory,
@@ -193,7 +192,6 @@ class LLMPolicyExecutor(Generic[CtxT]):
             response_schema_by_xml_tag=self.response_schema_by_xml_tag,
             tools=self.tools,
             tool_choice=tool_choice,
-            n_choices=1,
             proc_name=self.agent_name,
             call_id=call_id,
         )
@@ -212,10 +210,9 @@ class LLMPolicyExecutor(Generic[CtxT]):
 
         memory.update(completion.messages)
 
-        self._process_completion(
-            completion, print_messages=True, ctx=ctx, call_id=call_id
-        )
+        self._process_completion(completion, ctx=ctx, call_id=call_id)
 
+    # @task(name="call_tools")  # type: ignore
     async def call_tools(
         self,
         calls: Sequence[ToolCall],
@@ -237,13 +234,14 @@ class LLMPolicyExecutor(Generic[CtxT]):
 
         memory.update(tool_messages)
 
-        if ctx and ctx.printer:
+        if ctx.printer:
             ctx.printer.print_messages(
                 tool_messages, agent_name=self.agent_name, call_id=call_id
             )
 
         return tool_messages
 
+    # @task(name="call_tools")  # type: ignore
     async def call_tools_stream(
         self,
         calls: Sequence[ToolCall],
@@ -276,6 +274,7 @@ class LLMPolicyExecutor(Generic[CtxT]):
 
                 return final_answer_message
 
+    @task(name="enforce_final_answer")  # type: ignore
     async def _generate_final_answer(
         self, memory: LLMAgentMemory, ctx: RunContext[CtxT], call_id: str
     ) -> AssistantMessage:
@@ -283,7 +282,7 @@ class LLMPolicyExecutor(Generic[CtxT]):
             "Exceeded the maximum number of turns: provide a final answer now!"
         )
         memory.update([user_message])
-        if ctx and ctx.printer:
+        if ctx.printer:
             ctx.printer.print_messages(
                 [user_message], agent_name=self.agent_name, call_id=call_id
             )
@@ -299,6 +298,7 @@ class LLMPolicyExecutor(Generic[CtxT]):
 
         return final_answer_message
 
+    @task(name="generate_final_answer")  # type: ignore
     async def _generate_final_answer_stream(
         self, memory: LLMAgentMemory, ctx: RunContext[CtxT], call_id: str
     ) -> AsyncIterator[Event[Any]]:
@@ -309,7 +309,7 @@ class LLMPolicyExecutor(Generic[CtxT]):
         yield UserMessageEvent(
             proc_name=self.agent_name, call_id=call_id, data=user_message
         )
-        if ctx and ctx.printer:
+        if ctx.printer:
             ctx.printer.print_messages(
                 [user_message], agent_name=self.agent_name, call_id=call_id
             )
@@ -507,12 +507,7 @@ class LLMPolicyExecutor(Generic[CtxT]):
         return FinalAnswerTool()
 
     def _process_completion(
-        self,
-        completion: Completion,
-        *,
-        print_messages: bool = False,
-        ctx: RunContext[CtxT],
-        call_id: str,
+        self, completion: Completion, *, ctx: RunContext[CtxT], call_id: str
     ) -> None:
         ctx.completions[self.agent_name].append(completion)
         ctx.usage_tracker.update(
@@ -520,7 +515,7 @@ class LLMPolicyExecutor(Generic[CtxT]):
             completions=[completion],
             model_name=self.llm.model_name,
         )
-        if ctx.printer and print_messages:
+        if ctx.printer:
             usages = [None] * (len(completion.messages) - 1) + [completion.usage]
             ctx.printer.print_messages(
                 completion.messages,
