@@ -105,6 +105,14 @@ class Content(BaseModel):
     def from_text(cls, text: str) -> "Content":
         return cls(parts=[ContentPartText(data=text)])
 
+    @classmethod
+    def from_image(cls, image: ImageData) -> "Content":
+        return cls(parts=[ContentPartImage(data=image)])
+
+    @classmethod
+    def from_images(cls, images: Iterable[ImageData]) -> "Content":
+        return cls(parts=[ContentPartImage(data=image) for image in images])
+
     @classmethod
     def from_content_parts(cls, content_parts: Iterable[str | ImageData]) -> "Content":
         parts: list[ContentPart] = []
