@@ -3,6 +3,8 @@ from collections.abc import AsyncIterator, Sequence
 from itertools import pairwise
 from typing import Any, Generic, cast, final
 
+from grasp_agents.tracing_decorators import workflow
+
 from ..errors import WorkflowConstructionError
 from ..packet_pool import Packet
 from ..processors.base_processor import BaseProcessor
@@ -39,6 +41,7 @@ class SequentialWorkflow(WorkflowProcessor[InT, OutT, CtxT], Generic[InT, OutT,
                     f" {proc.name}"
                 )
 
+    @workflow(name="workflow_run")  # type: ignore
     @final
     async def run(
         self,
@@ -71,6 +74,7 @@ class SequentialWorkflow(WorkflowProcessor[InT, OutT, CtxT], Generic[InT, OutT,
 
         return cast("Packet[OutT]", packet)
 
+    @workflow(name="workflow_run")  # type: ignore
     @final
     async def run_stream(
         self,
